import QUB
import qubtree
from Dialogs import *

# 1: some sugar to mimic a button press and find out a button's settings
# 2: a framework for adding new buttons written in python

# 1:
# e.g. Buttons.Run('Modeling', 'MIL', {'use segments' : 'together'})    

def Run(category, name, tempPrefs=None):
    button = QUB.__dict__[str(category)][str(name)]

    storedPrefs = button.Properties()
    if not tempPrefs:
        tempPrefs = qubtree.Node()
    tempPrefs['DialogOnLClick'].data = 0
    button.Properties(tempPrefs)

    button()

    button.Properties(storedPrefs)

def GetSettings(category, btn):
    return QUB.Profiles[str(category)].active.clone()


# 2:
# This is a framework for adding buttons to the right-hand panel.
# You specify a function and some named settings, and it handles the
# 'edit settings' dialog for you.  Scroll to the bottom for discussion and example usage.


class ChoiceField:
    def __init__(self, name, caption, choices, default):
        self.name = name
        self.caption = caption
        self.choices = choices
        self.default = default

class TextField:
    def __init__(self, name, caption, default, acceptor=lambda x:x, format=str):
        self.name = name
        self.caption = caption
        self.default = default
        self.acceptor = acceptor
        self.format = format

def MakePromptField(field, value):
    if isinstance(field, ChoiceField):
        return DialogRadioGroup(field.name, field.caption, field.choices, str(value))
        # return PromptField(field.name, field.caption, str(value), field.choices)
    elif isinstance(field, TextField):
        return DialogText(field.name, field.caption, value, field.acceptor, field.format)
        # return PromptField(field.name, field.caption, str(value))


def PropTreeValue(propTree, name):
        data = propTree[name].data
        if len(data) == 0 or data.type == qubtree.QTR_TYPE_STRING:
            return str(data)
        else:
            return data[0]
    
def PropsToDict(properties, propTree):
    d = {}
    for prop in properties:
        d[prop.name] = PropTreeValue(propTree, prop.name)
    return d


# options include:
#   DataChannel=True,
#   DataList=True
#   FileList=True
#   Process=True
#   Threads=True
# for optional subpanels in properties dialog

class TaskButton:
    def __init__(self, name, func, properties, shortName, options, onDlogShow=lambda:None):
        self.name = name
        self.func = func
        self.shortName = shortName
        self.options = options
        self.onDlogShow = onDlogShow
        self.prof = QUB.Profiles[shortName]
        self.propTree = self.prof.active
        self.propertiesCaption = ''
        self.dlogOnLClick = 0
        self.dlogOnRClick = 1

        self.setupProperties(properties)

    def convertProperties(self, properties):
        for i in range(len(properties)):
            p = properties[i]
            if isinstance(p, TextField):
                properties[i] = DialogText(p.name, p.caption, p.acceptor(p.default), p.acceptor, p.format)
            elif isinstance(p, ChoiceField):
                properties[i] = DialogRadioGroup(p.name, p.caption, p.choices, p.default)

    def setupProperties(self, properties):
        self.convertProperties(properties)        
        self.properties = properties
        for prop in self.properties:
            propNode = self.propTree[prop.name]
            if len(propNode.data) == 0:
                propNode.data = prop.value
        
    def DoLClick(self):
        shouldRun = 1
        if self.dlogOnLClick:
            shouldRun = self.DoDlog()
        if shouldRun:
            self.func( PropsToDict(self.properties, self.propTree) )

    def DoRClick(self):
        shouldRun = 1
        if self.dlogOnRClick:
            shouldRun = self.DoDlog()
        if shouldRun:
            self.func( PropsToDict(self.properties, self.propTree) )

    def DoDlog(self):
        self.onDlogShow()
        response = QUB.TaskProperties( self.propertiesCaption,
                               ['Run', 'Close'], self.properties, self.shortName, self.options )
        return response == 'Run'

    def DoProperties(self, propNode):
        self.prof.setProperties(propNode)
        return self.propTree.clone()            


def AddTaskButton(name, func, shortName=None, dlogOnLClick=0, dlogOnRClick=1,
                  hint="", description="", propertiesCaption=None,
                  properties=[], category='Scripts', onDlogShow=lambda:None, **options):
#    print "Adding %s" % shortName
    
    if shortName:
        xShortName = shortName
    else:
        xShortName = name
    if not propertiesCaption:
        propertiesCaption = name

    task = TaskButton(name, func, properties, xShortName, options, onDlogShow)
    task.dlogOnLClick = dlogOnLClick
    task.dlogOnRClick = dlogOnRClick
    task.propertiesCaption = propertiesCaption

    QUB.__dict__[category].addButton(name, task.DoLClick, task.DoRClick, task.DoProperties,
                                     hint, description, xShortName)
        
    return task    



## The button's function takes one argument: a dictionary of named settings.
## This example merely prints all its settings.
## Uncomment it and restart QUB to try it out.
#
#def TestReport(settings):
#    for setname in settings.keys():
#        QUB.Report( setname + ': ' + str(settings[setname]) )
#    QUB.Report("-------------------------")
#
#test = AddTaskButton( 'TestReport', # button text
#                      TestReport,   # function
#                      'TRpt',       # short text for small buttons
#                      0,            # show settings dialog on left-click?
#                      1,            # show settings dialog on right-click?
#                      "Test Buttons.AddTaskButton",   # tool-tip
#                      "See PythonScripts/Buttons.py", # description for the "Edit Buttons" dialog
#                      "Please mess with these settings:", # caption for the settings dialog
#
## the last arg. is a list of settings.
## Settings have a name such as "abc" which is used for look-up in the function,
## and a caption such as "Pick one:" for display in the settings dialog.
## They can be either a ChoiceField (pick one of these strings) (in the example, 'b' is given as the default),
## or a TextField -- free response.
## The optional 4th argument to the TextField constructor is an "acceptor":
## a function which converts a string to whatever type your function expects,
## or throws an exception to reject the new setting.
## The default acceptor performs no conversions (it stays as a string).
#                      
#               [ChoiceField("abc", "Pick one:", ['a', 'b', 'c'], 'b'),
#                TextField("anything", "No limits:", ""),
#                TextField("number", "Any number:", "0", acceptFloat),
#                TextField("int", "Any integer:", "0", acceptInt),
#                TextField("bounded", "Float between 0 and 1:", "0", acceptFloatBetween(0, 1)),
#                TextField("boundedInt", "Int between 0 and 9:", "0", acceptIntBetween(0, 9)),
#                TextField("positive", "Positive int:", "1", acceptIntGreaterThan(0)),
#                TextField("negative", "Negative int:", "-1", acceptIntLessThan(0))] )
#

## Here's a smaller example that also shows how to put up your own dialogs:
#def Greet(props):
#    answer = QUB.Dialog(props['greeting'],                          # dialog caption
#                        ['Good', 'OK', 'Bad'],                      # buttons
#                        [Dialog.DialogText('Comments', 'Comments:', '')] )    # list of fields eg [DialogRadioGroup(name, caption, [list of choices], chosenvalue)]
#    QUB.Report( answer['button'] + ': ' + answer['Comments'] )
#
#AddTaskButton( 'Greet', Greet, 'Hi', 0, 1,
#               "Interact with the machine on a more human level",
#               "A simple example of how to add your own buttons via python.\nSee PythonScripts/Greet.py", "Customize the message:",
#               [TextField("greeting", "The greeting:", "Hello.  How are you today?", acceptString)] )

# p.s. now you can use the Dialogs.Dialog* family of dialog items instead of TextField and ChoiceField

